@@ -57,20 +57,24 @@ class UsersAPI(API):
         Whether to setup a startup event to connect databases.
     setup_shutdown : bool
         Whether to setup a shutdown event to close databases.
-    auth_router_auth: 'jwt' or 'cookie'
-        FastAPI Users auth method for the auth router. See `Auth router <https://fastapi-users.github.io/fastapi-users/configuration/routers/auth/>`_.
-    auth_router_kwargs : dict
-        Keyword arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.get_auth_router`. See `Auth router <https://fastapi-users.github.io/fastapi-users/configuration/routers/auth/>`_.
-    auth_router_include_kwargs : dict
-        Keyword arguments passed to :meth:`fastapi:fastapi.FastAPI.include_router` for the FastAPI Users auth router.
+    auth_router_jwt_kwargs : dict
+        Keyword arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.get_auth_router` using jwt auth. See `Auth router <https://fastapi-users.github.io/fastapi-users/configuration/routers/auth/>`_.
+    auth_router_jwt_include_kwargs : dict
+        Keyword arguments passed to :meth:`fastapi:fastapi.FastAPI.include_router` for the FastAPI Users auth router using jwt auth.
     auth_router_jwt_refresh : bool
         Whether to setup a token refresh route at ``auth_router_jwt_refresh_path``. This is setup according to the FastAPI Users `JWT authentication guide <https://fastapi-users.github.io/fastapi-users/configuration/authentication/jwt/>`_.
     auth_router_jwt_refresh_path : str
         Path to setup refresh token if ``auth_router_jwt_refresh`` is ``True``.
+    auth_router_cookie_kwargs : dict
+        Keyword arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.get_auth_router` using cookie auth. See `Auth router <https://fastapi-users.github.io/fastapi-users/configuration/routers/auth/>`_.
+    auth_router_cookie_include_kwargs : dict
+        Keyword arguments passed to :meth:`fastapi:fastapi.FastAPI.include_router` for the FastAPI Users auth router using cookie auth.
     register_router_kwargs : dict
         Keyword arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.get_register_router`. See `Register router <https://fastapi-users.github.io/fastapi-users/configuration/routers/register/>`_.
     register_router_include_kwargs : dict
         Keyword arguments passed to :meth:`fastapi:fastapi.FastAPI.include_router` for the FastAPI Users register router.
+    register_router_superuser : bool
+        Whether to limit registration control to superusers only. If ``dependencies`` are set in parameter ``register_router_include_kwargs``, then a superuser dependency will be added to the list if ``True``.
     verify_router_kwargs : dict
         Keyword arguments passed to :meth:`fastapi_users:fastapi_users.FastAPIUsers.get_verify_router`. See `Verify router <https://fastapi-users.github.io/fastapi-users/configuration/routers/verify/>`_.
     verify_router_include_kwarg : dict
@@ -158,10 +162,33 @@ class UsersAPI(API):
     ----------
     users_api : :class:`fastapi_users:fastapi_users.FastAPIUsers`
         Configured FastAPI Users object. See `FastAPIUsers object <https://fastapi-users.github.io/fastapi-users/configuration/routers/>`_.
-    _database_engine : :func:`sqlalchemy:sqlalchemy.create_engine`
-        Engine from parameter ``database_engine``.
-    _async_database : :class:`databases:databases.Database`
-        Async database from parameter ``async_database``.
+    _users_databases : dict
+        Dictionary of user database related objects:
+
+        * ``engine`` (:func:`sqlalchemy:sqlalchemy.create_engine`): Engine from parameter ``database_engine``.
+        * ``asynchronous`` (:class:`databases:databases.Database`): Async database from parameter ``async_database``.
+    
+    _users_functions : dict(func)
+        Dictionary of user related functions from auto-config:
+
+        * ``get_user_db`` (func): get_user_db function auto-configured or from parameter ``get_user_db``. See :func:`msdss_users_api.tools.create_user_db_func`.
+        * ``get_user_manager`` (func): get_user_manager function auto-configured or from parameter ``get_user_manager``. See :func`msdss_users_api.tools.create_user_manager_func`.
+    
+    _users_events : dict(func)
+        Dictionary of event functions registered from auto-config:
+
+        * ``startup``: startup function registered if ``setup_startup`` is ``True``
+        * ``shutdown``: shutdown function registered if ``setup_shutdown`` is ``True``
+    
+    _users_models : dict
+        Dictionary of user models for the users API:
+
+        * User (:class:`msdss_users_api.models.User`): see parameter ``User``.
+        * UserCreate (:class:`msdss_users_api.models.UserCreate`): see parameter ``UserCreate``.
+        * UserUpdate (:class:`msdss_users_api.models.UserUpdate`): see parameter ``UserUpdate``.
+        * UserDB (:class:`msdss_users_api.models.UserDB`): see parameter ``UserDB``.
+        * UserTable (:class:`msdss_users_api.models.UserTable`): see parameter ``UserTable``.
+        * UserManager (:class:`msdss_users_api.models.UserManager`): see parameter ``UserManager``.
 
     Author
     ------
@@ -204,22 +231,27 @@ class UsersAPI(API):
         enable_reset_password_router=True,
         enable_users_router=True,
         enable_jwt_auth=True,
-        enable_cookie_auth=False,
+        enable_cookie_auth=True,
         setup_startup=True,
         setup_shutdown=True,
-        auth_router_auth='jwt',
-        auth_router_kwargs={},
-        auth_router_include_kwargs={
+        auth_router_jwt_kwargs={},
+        auth_router_jwt_include_kwargs={
             'prefix': '/auth/jwt',
             'tags': ['auth']
         },
         auth_router_jwt_refresh=True,
         auth_router_jwt_refresh_path='/refresh',
+        auth_router_cookie_kwargs={},
+        auth_router_cookie_include_kwargs={
+            'prefix': '/auth',
+            'tags': ['auth']
+        },
         register_router_kwargs={},
         register_router_include_kwargs={
             'prefix': '/auth',
             'tags': ['auth']
         },
+        register_router_superuser=True,
         verify_router_kwargs={},
         verify_router_include_kwargs={
             'prefix': '/auth',
@@ -300,7 +332,8 @@ class UsersAPI(API):
         cookie_secret = secret if cookie_secret is None else cookie_secret
 
         # (UserAPI_db) Setup database connections
-        database_engine = Database(driver=driver, user=user, password=password, host=host, port=port, database=database)._connection if database_engine is None else database_engine
+        db = Database(driver=driver, user=user, password=password, host=host, port=port, database=database)
+        database_engine = db._connection if database_engine is None else database_engine
         async_database = databases.Database(str(database_engine.url)) if async_database is None else async_database
 
         # (UserAPI_auth) Setup auth for users
@@ -308,13 +341,11 @@ class UsersAPI(API):
         cookie_auth = CookieAuthentication(secret=cookie_secret, **cookie_kwargs) if cookie_secret and cookie_auth is None else cookie_auth
         
         # (UserAPI_auth_combine) Combine cookie and jwt auths if needed
-        jwt_auth = [jwt_auth] if not isinstance(jwt_auth, list) else jwt_auth
-        cookie_auth = [cookie_auth] if not isinstance(cookie_auth, list) else cookie_auth
         auth = []
-        if enable_jwt_auth:
-            auth = auth + jwt_auth
         if enable_cookie_auth:
-            auth = auth + cookie_auth
+            auth.append(cookie_auth)
+        if enable_jwt_auth:
+            auth.append(jwt_auth)
         auth = [a for a in auth if a is not None]
         
         # (UserAPI_func) Setup required functions
@@ -335,56 +366,87 @@ class UsersAPI(API):
         # (UsersAPI_router_auth) Add auth router
         if enable_auth_router:
 
-            # (UsersAPI_router_auth_method) Set method of auth for route
-            if auth_router_auth == 'jwt':
-                auth_router_auth = jwt_auth[0]
-            elif auth_router_auth == 'cookie':
-                auth_router_auth = cookie_auth[0]
+            # (UsersAPI_router_auth_jwt) Add jwt auth route
+            if enable_jwt_auth:
 
-            # (UsersAPI_router_auth_create) Create auth route
-            auth_router = users_api.get_auth_router(auth_router_auth, **auth_router_kwargs)
-            
-            # (UsersAPI_router_auth_refresh) Create auth refresh route
-            if auth_router_jwt_refresh:
-                @auth_router.post(auth_router_jwt_refresh_path)
-                async def refresh_jwt(response: Response, user=Depends(users_api.current_user(active=True))):
-                    return await auth_router_auth.get_login_response(user, response, UserManager)
+                # (UsersAPI_router_auth_jwt_method) Set method of auth for route
+                auth_jwt_router = users_api.get_auth_router(jwt_auth, **auth_router_jwt_kwargs)
+                
+                # (UsersAPI_router_auth_jwt_refresh) Create jwt auth refresh route
+                if auth_router_jwt_refresh:
+                    @auth_jwt_router.post(auth_router_jwt_refresh_path)
+                    async def refresh_jwt(response: Response, user=Depends(users_api.current_user(active=True))):
+                        return await jwt_auth.get_login_response(user, response, UserManager)
 
-            # (UsersAPI_router_auth_add) Add auth route
-            self.add_router(auth_router, **auth_router_include_kwargs)
+                # (UsersAPI_router_auth_jwt_add) Add jwt auth route
+                self.add_router(auth_jwt_router, **auth_router_jwt_include_kwargs)
+
+            # (UsersAPI_router_auth_cookie) Add cookie auth route
+            if enable_cookie_auth:
+
+                    # (UsersAPI_router_auth_cookie_method) Set method of auth for route
+                auth_cookie_router = users_api.get_auth_router(cookie_auth, **auth_router_cookie_kwargs)
+
+                # (UsersAPI_router_auth_cookie_add) Add auth route
+                self.add_router(auth_cookie_router, **auth_router_cookie_include_kwargs)
 
         # (UsersAPI_router_register) Add register router
         if enable_register_router:
-            self.add_router(users_api.get_register_router(**register_router_kwargs), **register_router_include_kwargs)
+            if register_router_superuser:
+                register_router_include_kwargs['dependencies'] = register_router_include_kwargs['dependencies'] if 'dependencies' in register_router_include_kwargs else []
+                register_router_include_kwargs['dependencies'].append(Depends(users_api.current_user(superuser=True)))
+            register_router = users_api.get_register_router(**register_router_kwargs)
+            self.add_router(register_router, **register_router_include_kwargs)
 
         # (UsersAPI_router_verify) Add verify router
         if enable_verify_router:
-            self.add_router(users_api.get_verify_router(**verify_router_kwargs), **verify_router_include_kwargs)
+            verify_router = users_api.get_verify_router(**verify_router_kwargs)
+            self.add_router(verify_router, **verify_router_include_kwargs)
         
         # (UsersAPI_router_reset) Add reset password router
         if enable_reset_password_router:
-            self.add_router(users_api.get_reset_password_router(**reset_password_router_kwargs), **reset_password_router_include_kwargs)
+            reset_password_router = users_api.get_reset_password_router(**reset_password_router_kwargs)
+            self.add_router(reset_password_router, **reset_password_router_include_kwargs)
 
-        # (UsersAPI_router_reset) Add reset password router
+        # (UsersAPI_router_users) Add users router
         if enable_users_router:
-            self.add_router(users_api.get_users_router(**users_router_kwargs), **users_router_include_kwargs)
+            users_router = users_api.get_users_router(**users_router_kwargs)
+            self.add_router(users_router, **users_router_include_kwargs)
+
+        # (UserAPI_attr) Add attributes
+        self.users_api = users_api
+        self.users_database = db
+        self._users_databases = dict(
+            engine = database_engine,
+            asynchronous = async_database
+        )
+        self._users_functions = dict(
+            get_user_db = get_user_db,
+            get_user_manager = get_user_manager
+        )
+        self._users_models = dict(
+            User = User,
+            UserCreate = UserCreate,
+            UserUpdate = UserUpdate,
+            UserDB = UserDB,
+            UserTable = UserTable,
+            UserManager = UserManager
+        )
+        self._users_events = {}
 
         # (UserAPI_events_startup) Setup app startup
         if setup_startup:
             @self.on('startup')
-            async def before_startup():
-                await async_database.connect()
+            async def startup():
+                await self._users_databases.asynchronous.connect()
+            self._users_events['startup'] = startup
 
         # (UserAPI_events_shutdown) Setup app shutdown
         if setup_shutdown:
             @self.on('shutdown')
-            async def during_shutdown():
-                await async_database.disconnect()
-
-        # (UserAPI_attr) Add attributes
-        self.users_api = users_api
-        self._database_engine = database_engine
-        self._async_database = async_database
+            async def shutdown():
+                await self._users_databases.asynchronous.disconnect()
+            self._users_events['shutdown'] = shutdown
 
     def get_current_user(self, *args, **kwargs):
         """
